#ifndef DISPLAY_SERVICE_MESSAGES_H
#define DISPLAY_SERVICE_MESSAGES_H

#define DS_MSG_TUTORIAL_INTRODUCTION "\
HLA Tutorial\n\n\
This tutorial gives a brief introduction in the usage of the High Level \
Architecture Interface Specification 1.3. HLA is a standard for distributed \
computer simulation systems. The middleware implementing HLA is called Runtime \
Infrastructure (RTI).\n\
The API is separated in so called RTI ambassador services and federate \
ambassador services. The RTI ambassador services are provided by an \
RTIambassador object and are handled like normal method invocations. In \
contrast, federate ambassador services have to be implemented within the \
federate and can be regarded as a callback mechanism.\n\
Furthermore the services are divided in different management areas. This \
tutorial shows and explains the correct usage of essential API calls of \
the management areas Federation Management, Declaration Management and \
Object Management.\
    "
    
/* Federation Management - RTI services */

#define DS_MSG_FEDERATION_MANAGEMENT "\
Federation Management\n\n\
Services of this management area coordinate federation-wide activities \
such as creating or destroying federations and joining or resigning a \
federation. Federation-wide save and restore activities and synchronization \
operations are also supported. \
    "

#define DS_MSG_CREATE_FEDERATION_EXECUTION "\
Federation Management -> Create Federation Execution\n\n\
The RTI ambassador service createFederationExecution is usally invoked by the \
first federate. This service creates the federation specifed by a name and a \
file holding the federation object model (FOM). \
    "
    
#define DS_MSG_CREATE_FEDERATION_EXECUTION_SRC "Example from file: controllerFederate.cc\n279:    /* create federation execution */\n\
280-    try {\n\
281-        rtiAmb.createFederationExecution(federationName.c_str(), \n\
282-					 fedFile.c_str());\n\
283-    } catch ( RTI::FederationExecutionAlreadyExists ) {\n\
284-        cout << \"Federation already created by another federate.\" \n\
285-	<< endl;\n\
286-    } catch ( RTI::Exception &e ) {\n\
287-        cerr << \"RTI exception: \" << e._name << \" [\"\n\
288-        << (e._reason ? e._reason : \"undefined\") << \"].\" << endl;\n\
289-    } catch ( ... ) {\n\
290-        cerr << \"Error: unknown non-RTI exception.\" << endl;\n\
291-    }\n\
"

#define DS_MSG_JOIN_FEDERATION_EXECUTION "\
Federation Management -> Join Federation Execution\n\n\
The RTI ambassador service joinFederationExecution is used by a federate to \
join an existing federation. Input parameters are his own name, the federation \
name of the federation it wants to join and a pointer to its federate \
ambassador implementation. \
    "

#define DS_MSG_JOIN_FEDERATION_EXECUTION_SRC "Example from file: controllerFederate.cc\n295:    /* join federation execution */\n\
296-    try {\n\
297-        rtiAmb.joinFederationExecution(federateName.c_str(),\n\
298-                                       federationName.c_str(),\n\
299-                                       &myFedAmb);\n\
300-    } catch ( RTI::Exception &e ) {\n\
301-        cerr << \"RTI exception: \" << e._name << \" [\"\n\
302-        << (e._reason ? e._reason : \"undefined\") << \"].\" << endl;\n\
303-    } catch ( ... ) {\n\
304-        cerr << \"Error: unknown non-RTI exception.\" << endl;\n\
305-    }\n\
"

#define DS_MSG_RESIGN_FEDERATION_EXECUTION "\
Federation Management -> Resign Federation Execution\n\n\
The RTI ambassador service resignFederationExecution is used by a federate \
when leaving the federation. The required ResignAction parameter resolves \
ownership of any attributes owned by the federate. Allowed values are: \
RELEASE_ATTRIBUTES, DELETE_OBJECT, \
DELETE_OBJECTS_AND_RELEASE_ATTRIBUTES (common practice), NO_ACTION. \
    " 

#define DS_MSG_RESIGN_FEDERATION_EXECUTION_SRC "Example from file: controllerFederate.cc\n568:    /* resign federation execution */\n\
569-    try {\n\
570-        rtiAmb.resignFederationExecution(\n\
571-            RTI::DELETE_OBJECTS_AND_RELEASE_ATTRIBUTES);\n\
572-    } catch ( RTI::Exception &e ) {\n\
573-        cerr << \"RTI exception: \" << e._name << \" [\"\n\
574-        << (e._reason ? e._reason : \"undefined\") << \"].\" << endl;\n\
575-    } catch ( ... ) {\n\
576-        cerr << \"Error: unknown non-RTI exception.\" << endl;\n\
577-    }\n\
"

#define DS_MSG_DESTROY_FEDERATION_EXECUTION "\
Federation Management -> Destroy Federation Execution\n\n\
The RTI ambassador service destroyFederationExecution is usally invoked \
by the last federate. This service destroys the current federation \
execution. \
    " 

#define DS_MSG_DESTROY_FEDERATION_EXECUTION_SRC "Example from file: controllerFederate.cc\n581:    /* destroy federation execution */\n\
582-    try {\n\
583-        rtiAmb.destroyFederationExecution(federationName.c_str());\n\
584-    } catch (RTI::FederatesCurrentlyJoined) {\n\
585-        cout << \"Federates currently joined.\" << endl;\n\
586-    } catch ( RTI::Exception &e ) {\n\
587-        cerr << \"RTI exception: \" << e._name << \" [\"\n\
588-        << (e._reason ? e._reason : \"undefined\") << \"].\" << endl;\n\
589-    } catch ( ... ) {\n\
590-        cerr << \"Error: unknown non-RTI exception.\" << endl;\n\
591-    }\n\
"

/* Declaration Management - RTI services */

#define DS_MSG_DECLARATION_MANAGEMENT "\
Declaration Management\n\n \
HLA communication relies on an implicit data exchange mechanism. That's why \
federates have to declare their intention to generate or receive information \
in their initialization phase according to their FOM resp. their SOM. \
Declaration Management can also be regarded as interest management. \
    "

#define DS_MSG_PUBLISH_OBJECT_CLASS "\
Declaration Management -> Publish Object Class\n\n \
When invoking this service the federate announces that it can produce \
information for the specified attributes within the AttributeHandleSet class \
in its future execution. \
    "

#define DS_MSG_PUBLISH_OBJECT_CLASS_SRC "Example from file: controllerFederate.cc\n362:    /* publish u */\n\
363-    try {\n\
364-        rtiAmb.publishObjectClass(twoLevelContrID, *attrU);\n\
365-    } catch ( RTI::Exception &e ) {\n\
366-        cerr << \"RTI exception: \" << e._name << \" [\"\n\
367-        << (e._reason ? e._reason : \"undefined\") << \"].\" << endl;\n\
368-    } catch ( ... ) {\n\
369-        cerr << \"Error: unknown non-RTI exception.\" << endl;\n\
370-    }\n\
"

#define DS_MSG_SUBSCRIBE_TO_OBJECT_CLASS "\
Declaration Management -> Subscribe To Object Class\n\n \
When invoking this service the federate announces that it is interested in \
receiving updates for the specified attributes within the AttributeHandleSet \
class in its future execution. \
    "

#define DS_MSG_SUBSCRIBE_TO_OBJECT_CLASS_SRC "Example from file: controllerFederate.cc\n374:    /* subscribe to y */\n\
375-    try {\n\
376-        rtiAmb.subscribeObjectClassAttributes(twoLevelContrID, *attrY);\n\
377-    } catch ( RTI::Exception &e ) {\n\
378-        cerr << \"RTI exception: \" << e._name << \" [\"\n\
379-        << (e._reason ? e._reason : \"undefined\") << \"].\" << endl;\n\
380-    } catch ( ... ) {\n\
381-        cerr << \"Error: unknown non-RTI exception.\" << endl;\n\
382-    }\n\
"

/* Object Management - RTI services */

#define DS_MSG_OBJECT_MANAGEMENT "\
Object Management\n\n \
This part of the interface specification deals with deletion, instanciation, \
modification and registration of object instances. The necessary data exchange \
mechanisms, especially updating and reflecting attributes and sending and \
receiving interactions are part of this management area. \
    "

#define DS_MSG_REGISTER_OBJECT_INSTANCE "\
Object Management -> Register Object Instance\n\n \
This service creates a new object instance. Other federates are informed \
through the federate service discoverObjectInstance. \
    "

#define DS_MSG_REGISTER_OBJECT_INSTANCE_SRC "Example from file: controllerFederate.cc\n421:    /* register object */\n\
422-    RTI::ObjectHandle objInstID_u;\n\
423-\n\
424-    try {\n\
425-        objInstID_u = rtiAmb.registerObjectInstance(twoLevelContrID, \"u\");\n\
426-    } catch ( RTI::Exception &e ) {\n\
427-        cerr << \"RTI exception: \" << e._name << \" [\"\n\
428-        << (e._reason ? e._reason : \"undefined\") << \"].\" << endl;\n\
429-    } catch ( ... ) {\n\
430-        cerr << \"Error: unknown non-RTI exception.\" << endl;\n\
431-    }\n\
"

#define DS_MSG_UPDATE_ATTRIBUTE_VALUES "\
Object Management -> Update Attribute Values\n\n \
The federate informs interested federates about object updates. Interested \
federates will receive the updates by a reflectAttributeValues callback. \
    "

#define DS_MSG_UPDATE_ATTRIBUTE_VALUES_SRC "Example from file: controllerFederate.cc\n505:        /* update attribute */\n\
506-        ahvps -> add(uID, (char*)&(u), sizeof(double));\n\
507-\n\
508-        try {\n\
509-            rtiAmb.updateAttributeValues(objInstID_u, *ahvps, \"u\");\n\
510-        } catch ( RTI::Exception &e ) {\n\
511-            cerr << \"RTI exception: \" << e._name << \" [\"\n\
512-            << (e._reason ? e._reason : \"undefined\") << \"].\" << endl;\n\
513-        } catch ( ... ) {\n\
514-            cerr << \"Error: unknown non-RTI exception.\" << endl;\n\
515-        }\n\
"

/* Object Management - Federate services */

#define DS_MSG_DISCOVER_OBJECT_INSTANCE "\
Object Management -> Discover Object Instance\n\n \
This federate service informs the federate of a new object instance. Federates \
are only informed about new objects if they have specified their interests in \
a former subscription invocation. In this case, we are informed about a new \
instance of the output variable y. \
    "

#define DS_MSG_DISCOVER_OBJECT_INSTANCE_SRC "Example from file: controllerFederate.cc\n178:    discoverObjectInstance (RTI::ObjectHandle theObject,\n\
179-                            RTI::ObjectClassHandle theObjectClass,\n\
180-                            const char *theObjectName)\n\
181-    throw ( RTI::CouldNotDiscover,\n\
182-            RTI::ObjectClassNotKnown,\n\
183-            RTI::FederateInternalError) {\n\
184-        cout << \"discoverObjInst \" << theObjectName << endl;\n\
185-        if (!strcmp(theObjectName,\"y\")) {\n\
186-            discovY = true;\n\
187-        }\n\
188-    }\n\
"

#define DS_MSG_REFLECT_ATTRIBUTE_VALUES "\
Object Management -> Reflect Attribute Values\n\n \
This federate service informs the federate of new state updates for attributes \
it is currently interesting in. \
    "

#define DS_MSG_REFLECT_ATTRIBUTE_VALUES_SRC "Example from file: controllerFederate.cc\n200:    reflectAttributeValues (RTI::ObjectHandle theObject,\n\
201-                            const RTI::AttributeHandleValuePairSet&\n\
202-                            theAttributes,\n\
203-                            const char *theTag)\n\
204-    throw ( RTI::ObjectNotKnown,\n\
205-            RTI::AttributeNotKnown,\n\
206-            RTI::FederateOwnsAttributes,\n\
207-            RTI::FederateInternalError) {\n\
208-        RTI::ULong length;\n\
209-        char *attrValue;\n\
210-\n\
211-        /*\n\
212-        * common implementation scheme is to iterate through the\n\
213-        * AttributeHandleValuePairSet.\n\
214-        */\n\
215-        for (unsigned int i=0; i<theAttributes.size(); i++) {\n\
216-\n\
217-            RTI::AttributeHandle handle = theAttributes.getHandle(i);\n\
218-            length = theAttributes.getValueLength(i);\n\
219-            attrValue = new char[length];\n\
220-            theAttributes.getValue(i, attrValue, length);\n\
221-\n\
222-            if (!strcmp(theTag,\"y\")) {\n\
223-                y = *(reinterpret_cast<double*>(attrValue));\n\
224-                newY = true;\n\
225-            }\n\
226-\n\
227-            delete[] attrValue;\n\
228-        }\n\
229-    }\n\
"

/* Ancillary Services */

#define DS_MSG_GET_OBJECT_CLASS_HANDLE "\
Ancillary Service -> Get Object Class Handle\n\n \
This service returns a numeric handle for the named object class \
according to the FOM resp. SOM. \
    "

#define DS_MSG_GET_OBJECT_CLASS_HANDLE_SRC "Example from file: controllerFederate.cc\n315:    /* get object class handle */\n\
316-    RTI::ObjectClassHandle twoLevelContrID;\n\
317-\n\
318-    try {\n\
319-        twoLevelContrID = rtiAmb.getObjectClassHandle(\"TwoLevelControllerClass\");\n\
320-    } catch ( RTI::Exception &e ) {\n\
321-        cerr << \"RTI exception: \" << e._name << \" [\"\n\
322-        << (e._reason ? e._reason : \"undefined\") << \"].\" << endl;\n\
323-    } catch ( ... ) {\n\
324-        cerr << \"Error: unknown non-RTI exception.\" << endl;\n\
325-    }\n\
"

#define DS_MSG_GET_ATTRIBUTE_HANDLE "\
Ancillary Service -> Get Attribute Handle\n\n \
This service returns a numeric handle for the named attribute according to \
the FOM resp. SOM. \
    "

#define DS_MSG_GET_ATTRIBUTE_HANDLE_SRC "Example from file: controllerFederate.cc\n329:    /* get attribute handle */\n\
330-    RTI::AttributeHandle uID, yID;\n\
331-    try {\n\
332-        uID = rtiAmb.getAttributeHandle(\"U\", twoLevelContrID);\n\
333-    } catch ( RTI::Exception &e ) {\n\
334-        cerr << \"RTI exception: \" << e._name << \" [\"\n\
335-        << (e._reason ? e._reason : \"undefined\") << \"].\" << endl;\n\
336-    } catch ( ... ) {\n\
337-        cerr << \"Error: unknown non-RTI exception.\" << endl;\n\
338-    }\n\
"

#define DS_MSG_ADD_TO_ATTRIBUTE_HANDLE_SET "\
Ancillary Service -> Add attribute handles to attribute handle set\n\n \
The code below shows how to add attributes to the AttributeHandleSet class. \
    "

#define DS_MSG_ADD_TO_ATTRIBUTE_HANDLE_SET_SRC "Example from file: controllerFederate.cc\n351:    /* add attribute handle to AttributeHandleSet */\n\
352-    auto_ptr<RTI::AttributeHandleSet>\n\
353-    attrU(RTI::AttributeHandleSetFactory::create(1));\n\
354-    auto_ptr<RTI::AttributeHandleSet>\n\
355-    attrY(RTI::AttributeHandleSetFactory::create(1));\n\
356-\n\
357-    attrU->add(uID);\n\
358-    attrY->add(yID);\n\
"

#define DS_MSG_TICK "\
Ancillary Service -> Tick\n\n \
Tick is used to provide computation time to the LRC. Here we tick until we \
get the discoverObjectInstance callback. \
    "

#define DS_MSG_TICK_SRC "Example from file: controllerFederate.cc\n436:    /* discover y */\n\
437-    cout << \"Wait for second federate.\" << endl;\n\
438-    while (!myFedAmb.getDiscoverY()) {\n\
439-\n\
440-        try {\n\
441-            rtiAmb.tick();\n\
442-        } catch ( RTI::Exception &e ) {\n\
443-            cerr << \"RTI exception: \" << e._name << \" [\"\n\
444-            << (e._reason ? e._reason : \"undefined\") << \"].\" << endl;\n\
445-        } catch ( ... ) {\n\
446-            cerr << \"Error: unknown non-RTI exception.\" << endl;\n\
447-        }\n\
448-\n\
449-        sleep(1);\n\
450-    }\n\
451-    myFedAmb.setDiscoverY(false);\n\
"

#ifdef _WIN32
#define DS_CLEAR_CONSOLE_COMMAND "cls"
#else
#define DS_CLEAR_CONSOLE_COMMAND "clear"
#endif
#endif
